#****************************************************************
#
#                       ＭＥＡＴＥＳＴ
#                     www.meatest.com
#
#                           M160
#
#****************************************************************

import sys
import socket
import time

# Configuration variables
IPaddr = '192.168.1.100'    # Set IPv4 address of the M160 here
tcpPort = 22                # Set TCP port here, default port is 22
test_voltage = 3            # Set the voltage to be tested here, the maximum possible value for testing is 3
LOW_TO = 2                  # Set communication timeout here, the recommended default value is 2.

class Calib:
    """ Manages communication with Meatest M160 via ethernet using TCP sockets. """

    def __init__(self, calibIPaddr, tcpPort):  
        self.IPaddr = calibIPaddr
        self.tcpPort = tcpPort
        self.socket = None
        self.LOW_TO = LOW_TO
        if not self.openConnection():
            raise Exception("Device unreachable")
        self.initialized = True

    def openConnection(self):
        """ Initializes TCP connection and sets up open socket handler """
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.socket.settimeout(self.LOW_TO)
            self.socket.connect((self.IPaddr, self.tcpPort))
            print(f"Connected to {self.IPaddr}:{self.tcpPort}")
            return True
        except Exception as e:
            print(f"Failed to connect: {e}")
            return False

    def send_command(self, command):
        """ Sends a command to the M160 device and returns the response. """
        if not self.initialized:
            raise Exception("Connection not initialized")
        try:
            if isinstance(command, str):
                command = command.encode('utf-8')
            print(f"Sending command: {command.decode('utf-8')}")
            self.socket.sendall(command + b'\n')
            response = self.receive_response()
            return response
        except Exception as e:
            print(f"Error sending command: {e}")
            return None
       
    def receive_response(self):
        """ Receives a response from the device. """
        response = b''
        while True:
            try:     
                response += self.socket.recv(1024)                    
            except socket.timeout:
                break
        return response.decode('utf-8')        
        
    def clearStatus(self):
        """ Clears device status byte and error queue """
        scpiCmd = b'*CLS'
        self.send_command(scpiCmd)

    def waitCmdComplete(self):
        """ Sends *WAI: Prevents the device from processing further commands but DOES NOT HALT the PC program """
        scpiCmd = b'*WAI'
        self.send_command(scpiCmd)

    def isReady(self, opcTO):
        """ Sends *OPC?: Prevents the device from processing further commands and HALTS the PC program """
        scpiCmd = b'*OPC?'
        self.send_command(scpiCmd)

    def reset(self):
        """ Sends *RST to set the device to default state """
        scpiCmd = b'*RST'
        self.send_command(scpiCmd)

    def enableRemote(self):
        """ Sends SYST:REM to switch the device into Remote Mode (M160 will respond only to *IDN? otherwise) """
        scpiCmd = b'SYST:REM'
        self.send_command(scpiCmd)

    def readErr(self):
        """ Sends SYST:ERR?: Reads device's error queue """
        scpiCmd = b'SYST:ERR?'
        error = self.send_command(scpiCmd)
        return error.strip() if error else None

    def identity(self):
        """ Sends *IDN?: Reads device info inc. Make, Model, SN, FW version """
        scpiCmd = b'*IDN?'
        idn = self.send_command(scpiCmd)
        return idn.strip() if idn else None

    def setVoltage(self, vdc):
        """ Sets output voltage """
        scpiCmd = f'VDC {vdc}'.encode('UTF-8')
        self.send_command(scpiCmd)

    def outputOn(self):
        """ Sends OUTP:STAT 1 to turn the output on """
        scpiCmd = b'OUTP:STAT 1'
        self.send_command(scpiCmd)

    def outputOff(self):
        """ Sends OUTP:STAT 0 to turn the output off """
        scpiCmd = b'OUTP:STAT 0'
        self.send_command(scpiCmd)

    def close(self):
        """ Closes the socket connection. """
        if self.socket:
            self.socket.close()
            print("Connection closed.")

if __name__ == "__main__":
    # Connection initialization code block
    calibrator = Calib(IPaddr, tcpPort)
    calibrator.enableRemote()
    calibrator.reset()
    calibrator.isReady(0.5)
    calibrator.clearStatus()

    # This procedure prints device info, steps through voltages 1 V, 2 V and 3 V and prints errors along the way (if any)
    print(calibrator.identity())
    for V in range(1, test_voltage + 1):
        calibrator.setVoltage(V)
        calibrator.outputOn()
        time.sleep(1)
        calibrator.outputOff()
        errors = calibrator.readErr()
        print(f"Error after setting voltage {V}V: {errors}")

    # Connection termination code block
    calibrator.close()
    print()
    input("Test completed successfully. Press Enter to exit.")